# 物料清单

## 物料清单列表

```shell
curl -H "Authorization: Token token=<token>, uid=<uid>" \
  http://example.com/api/boms.json
```

```json
{
    "status": {
        "code": 200,
        "message": "OK"
    },
    "boms": [
        {
            "id": 234,
            "status": "submitted",
            "number": "BOM0039",
            "note": "",
            "created_at": "2019-04-10",
            "print_count": 0
        }
    ]
}
```

* HTTP请求

    `GET /api/boms.json`

* 请求参数

参数名    | 是否必填   | 默认值   | 字段说明
    -------- | --------- | ------  | -----
    page       | 无        | 1      | 第几页
	per       | 无        | 15      | 每页返回数
	keyword       | 无        | 无      | 关键词
	keyword_type       | 无        | 无      | 关键词类型(bill_items.name: 成品名称; bill_items.number: 成品编号; bills.number: 物料清单编号)
	finished_product_id       | 无        | all      | 成品产品
	created_at       | 无        | 本月      | 创建时间

* 返回字段说明

返回值字段 | 字段类型 | 字段说明
    -------- | --------- | -----------
    id       | integer | ID
    status   | string  | 状态
    number   | string  | 编号
    note     | string  | 备注
    created_at | string | 创建日期
    print_count | integer | 打印次数

## 新增物料清单

```shell
curl -i -X POST --header "Authorization: Token token=<user_token>, uid=<uid>" \
  http://example.com/api/boms.json
```

```json
{
  "status": {
    "code": "200",
    "message": "OK"
  },
  "bom": {
    "id": 57
  }
}
```

* HTTP请求

    `POST /api/boms.json`

* 请求参数

参数名    | 是否必填   | 默认值   | 字段说明
    -------- | --------- | ------  | -----
    status           | 是        | 无      | 审批状态(draft[草稿] approving[待审批] submitted[已提交] rejected[已驳回] passed[审批通过] deprecated[已作废])
    number           | 是        | 无      | 编号
    note             | 否        | 无      | 备注
    total_quantity   | 否        | 无      | 总数量
    total_base_quantity | 否        | 无      | 基本数量合计（开启多单位时候对应的基本单位的数量，未开启时候跟数量一致）
    attachments_attributes[0][dn_key] | 否 | 无 | 七牛存储的key
    attachments_attributes[0][key] | 否 | 无 | 同上
    attachments_attributes[0][filename] | 否 | 无 | 文件名
    attachments_attributes[0][url] | 否 | 无 | 文件URL
    attachments_attributes[0][size] | 否 | 无 | 文件大小
    finished_item_attributes[product_attr_group_id] | 是 | 无 | 产品属性组ID
    finished_item_attributes[product_unit_id] | 是 | 无 | 产品单位ID
    finished_item_attributes[product_id]  | 是 | 无 | 产品ID
    finished_item_attributes[warehouse_id]  | 是 | 无 | 仓库ID
    finished_item_attributes[warehouse_name]  | 是 | 无 | 仓库名称
    finished_item_attributes[name]  | 是 | 无 | 产品项名称
    finished_item_attributes[number] | 是 | 无 | 产品项编号
    finished_item_attributes[attr] | 否 | 无 | 产品项属性名称
    finished_item_attributes[spec]  | 否 | 无 | 产品项规格
    finished_item_attributes[unit_name]  | 是 | 无 | 单位名称
    finished_item_attributes[quantity]  | 是 | 无 | 产品项数量
    finished_item_attributes[base_unit] | 否 | 无 | 基本单位
    finished_item_attributes[base_quantity] | 否 | 无 | 基本数量（开启多单位时候基本数量等于基本单位与当前单位的换算率 * 当前数量；未开启多单位时候和当前数量一致）
    finished_item_attributes[deputy_unit_quantity] | 否 | 无 | 副单位数（需要开启多单位）
    finished_item_attributes[note]  | 是 | 无 | 产品项备注
    finished_item_attributes[conversion]  | 是 | 无 | 零件与成品的换算关系
    finished_item_attributes[modified] | 是 | 无 | 最后修改字段
    finished_item_attributes[_destroy]  | 否 | false | 产品项是否删除
    part_item_attributes[0][product_attr_group_id] | 是 | 无 | 产品属性组ID
    part_item_attributes[0][product_unit_id] | 是 | 无 | 产品单位ID
    part_item_attributes[0][product_id]  | 是 | 无 | 产品ID
    part_item_attributes[0][warehouse_id]  | 是 | 无 | 仓库ID
    part_item_attributes[0][warehouse_name]  | 是 | 无 | 仓库名称
    part_item_attributes[0][name]  | 是 | 无 | 产品项名称
    part_item_attributes[0][number] | 是 | 无 | 产品项编号
    part_item_attributes[0][attr] | 否 | 无 | 产品项属性名称
    part_item_attributes[0][spec]  | 否 | 无 | 产品项规格
    part_item_attributes[0][unit_name]  | 是 | 无 | 单位名称
    part_item_attributes[0][quantity]  | 是 | 无 | 产品项数量
    part_item_attributes[0][base_unit] | 否 | 无 | 基本单位
    part_item_attributes[0][base_quantity] | 否 | 无 | 基本数量（开启多单位时候基本数量等于基本单位与当前单位的换算率 * 当前数量；未开启多单位时候和当前数量一致）
    part_item_attributes[0][deputy_unit_quantity] | 否 | 无 | 副单位数（需要开启多单位）
    part_item_attributes[0][note]  | 是 | 无 | 产品项备注
    part_item_attributes[0][conversion]  | 是 | 无 | 零件与成品的换算关系
    part_item_attributes[0][modified] | 是 | 无 | 最后修改字段
    part_item_attributes[0][_destroy]  | 否 | false | 产品项是否删除
    

* 返回字段说明

返回值字段 | 字段类型 | 字段说明
    -------- | --------- | -----------
    id       | string | ID

## 物料清单详情

```shell
curl -H "Authorization: Token token=<token>, uid=<uid>" \
  http://example.com/api/boms/:id.json
```

```json
{
    "status": {
        "code": 200,
        "message": "OK"
    },
    "boms": {
        "id": 234,
        "status": "submitted",
        "number": "BOM0039",
        "note": "",
        "created_at": "2019-04-10",
        "print_count": 0,
        "finished_item": {
            "id": 571,
            "product_id": 401,
            "product_attr_group_id": 719,
            "warehouse_id": 25,
            "product_unit_id": 684,
            "category": "finished",
            "name": "CP0224",
            "number": "CP0224",
            "attr": "",
            "spec": "",
            "warehouse_name": "名字很长很长很长的仓库",
            "unit_name": "车",
            "quantity": "1.0",
            "base_unit": "台",
            "base_quantity": "200.0",
            "deputy_unit_quantity": "1车",
            "note": "",
            "conversion": "1.0"
        },
        "part_items": [
            {
                "id": 572,
                "product_id": 399,
                "product_attr_group_id": 717,
                "warehouse_id": 24,
                "product_unit_id": 678,
                "category": "part",
                "name": "CP0221",
                "number": "CP0221",
                "attr": "",
                "spec": "",
                "warehouse_name": "测试仓库11",
                "unit_name": "台",
                "quantity": "2.1",
                "base_unit": "台",
                "base_quantity": "2.1",
                "deputy_unit_quantity": "2.1台",
                "note": "",
                "conversion": "2.1"
            }
        ]
    }
}
```

* HTTP请求

    `GET /api/boms/:id.json`
    
* 请求参数

    无
    
* 返回字段说明

返回值字段 | 字段类型 | 字段说明
    -------- | --------- | -----------
    id       | integer | ID
    status   | string  | 审批状态(draft[草稿] approving[待审批] submitted[已提交] rejected[已驳回] passed[审批通过] deprecated[已作废])
    number   | string  | 编号
    note     | string  | 备注
    created_at | string | 创建日期
    print_count | integer | 打印次数
    finished_item[id] | integer | 成品ID
    finished_item[product_id] | integer | 产品ID
    finished_item[product_attr_group_id] | integer | 产品属性组ID 
    finished_item[product_unit_id] | integer | 产品单位ID
    finished_item[warehouse_id] | integer | 仓库ID
    finished_item[category] | string | 类别（finished）
    finished_item[name] | string | 产品名称
    finished_item[number] | string | 产品编号 
    finished_item[attr] | string | 产品属性名称
    finished_item[spec] | string | 产品规格
    finished_item[warehouse_name] | string | 仓库名称
    finished_item[unit_name] | string | 单位名称
    finished_item[quantity] | string | 数量
    finished_item[base_unit] | string | 基本单位名称
    finished_item[base_quantity] | string | 基本数量（开启多单位时候基本数量等于基本单位与当前单位的换算率 * 当前数量；未开启多单位时候和当前数量一致）
    finished_item[deputy_unit_quantity] | string | 副单位数（需要开启多单位）
    finished_item[note] | string | 备注
    finished_item[conversion] | string | 换算关系
    part_items[0][id] | integer | 零件ID
    part_items[0][product_id] | integer | 产品ID
    part_items[0][product_attr_group_id] | integer | 产品属性组ID 
    part_items[0][product_unit_id] | integer | 产品单位ID
    part_items[0][warehouse_id] | integer | 仓库ID
    part_items[0][category] | string | 类别（part）
    part_items[0][name] | string | 产品名称
    part_items[0][number] | string | 产品编号 
    part_items[0][attr] | string | 产品属性名称
    part_items[0][spec] | string | 产品规格
    part_items[0][warehouse_name] | string | 仓库名称
    part_items[0][unit_name] | string | 单位名称
    part_items[0][quantity] | string | 数量
    part_items[0][base_unit] | string | 基本单位名称
    part_items[0][base_quantity] | string | 基本数量（开启多单位时候基本数量等于基本单位与当前单位的换算率 * 当前数量；未开启多单位时候和当前数量一致）
    part_items[0][deputy_unit_quantity] | string | 副单位数（需要开启多单位）
    part_items[0][note] | string | 备注
    part_items[0][conversion] | string | 换算关系
    attachments[0][id] | integer | 附件ID
    attachments[0][filename] | string | 附件文件名
    attachments[0][url] | string | 附件URL

## 编辑物料清单

```shell
curl -i -X PUT --header "Authorization: Token token=<user_token>, uid=<uid>" \
  http://example.com/api/boms/:id.json
```

```json
{
  "status": {
    "code": "200",
    "message": "OK"
  },
  "bom": {
    "id": 57,
  }
}
```

* HTTP请求

    `PUT /api/boms/:id.json`

* 请求参数

参数名    | 是否必填   | 默认值   | 字段说明
    -------- | --------- | ------  | -----
    status           | 是        | 无      | 审批状态(draft[草稿] approving[待审批] submitted[已提交] rejected[已驳回] passed[审批通过] deprecated[已作废])
    number           | 是        | 无      | 编号
    note             | 否        | 无      | 备注
    total_quantity   | 否        | 无      | 总数量
    total_base_quantity | 否        | 无      | 基本数量合计（开启多单位时候对应的基本单位的数量，未开启时候跟数量一致）
    attachments_attributes[0][id] | 否 | 无 | 附件ID
    attachments_attributes[0][dn_key] | 否 | 无 | 七牛存储的key
    attachments_attributes[0][key] | 否 | 无 | 同上
    attachments_attributes[0][filename] | 否 | 无 | 文件名
    attachments_attributes[0][url] | 否 | 无 | 文件URL
    attachments_attributes[0][size] | 否 | 无 | 文件大小
    finished_item_attributes[id] | 是 | 无 | 成品ID
    finished_item_attributes[product_attr_group_id] | 是 | 无 | 产品属性组ID
    finished_item_attributes[product_unit_id] | 是 | 无 | 产品单位ID
    finished_item_attributes[product_id]  | 是 | 无 | 产品ID
    finished_item_attributes[warehouse_id]  | 是 | 无 | 仓库ID
    finished_item_attributes[warehouse_name]  | 是 | 无 | 仓库名称
    finished_item_attributes[name]  | 是 | 无 | 产品项名称
    finished_item_attributes[number] | 是 | 无 | 产品项编号
    finished_item_attributes[attr] | 否 | 无 | 产品项属性名称
    finished_item_attributes[spec]  | 否 | 无 | 产品项规格
    finished_item_attributes[unit_name]  | 是 | 无 | 单位名称
    finished_item_attributes[quantity]  | 是 | 无 | 产品项数量
    finished_item_attributes[base_unit] | 否 | 无 | 基本单位
    finished_item_attributes[base_quantity] | 否 | 无 | 基本数量（开启多单位时候基本数量等于基本单位与当前单位的换算率 * 当前数量；未开启多单位时候和当前数量一致）
    finished_item_attributes[deputy_unit_quantity] | 否 | 无 | 副单位数（需要开启多单位）
    finished_item_attributes[note]  | 是 | 无 | 产品项备注
    finished_item_attributes[conversion]  | 是 | 无 | 零件与成品的换算关系
    finished_item_attributes[modified] | 是 | 无 | 最后修改字段
    finished_item_attributes[_destroy]  | 否 | false | 产品项是否删除
    part_item_attributes[0][id] | 是 | 无 | 零件ID
    part_item_attributes[0][product_attr_group_id] | 是 | 无 | 产品属性组ID
    part_item_attributes[0][product_unit_id] | 是 | 无 | 产品单位ID
    part_item_attributes[0][product_id]  | 是 | 无 | 产品ID
    part_item_attributes[0][warehouse_id]  | 是 | 无 | 仓库ID
    part_item_attributes[0][warehouse_name]  | 是 | 无 | 仓库名称
    part_item_attributes[0][name]  | 是 | 无 | 产品项名称
    part_item_attributes[0][number] | 是 | 无 | 产品项编号
    part_item_attributes[0][attr] | 否 | 无 | 产品项属性名称
    part_item_attributes[0][spec]  | 否 | 无 | 产品项规格
    part_item_attributes[0][unit_name]  | 是 | 无 | 单位名称
    part_item_attributes[0][quantity]  | 是 | 无 | 产品项数量
    part_item_attributes[0][base_unit] | 否 | 无 | 基本单位
    part_item_attributes[0][base_quantity] | 否 | 无 | 基本数量（开启多单位时候基本数量等于基本单位与当前单位的换算率 * 当前数量；未开启多单位时候和当前数量一致）
    part_item_attributes[0][deputy_unit_quantity] | 否 | 无 | 副单位数（需要开启多单位）
    part_item_attributes[0][note]  | 是 | 无 | 产品项备注
    part_item_attributes[0][conversion]  | 是 | 无 | 零件与成品的换算关系
    part_item_attributes[0][modified] | 是 | 无 | 最后修改字段
    part_item_attributes[0][_destroy]  | 否 | false | 产品项是否删除
    

* 返回字段说明

返回值字段 | 字段类型 | 字段说明
    -------- | --------- | -----------
    id       | string | ID